@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 23951 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2010-01-20 00:51:49 -0600 (Wed, 20 Jan 2010) $
  */
 
 #ifndef PTLIB_HTML_H
@@ -901,7 +901,32 @@ class PHTML : public PStringStream
     };
 
 
-    class InputRange : public InputField {
+    class InputNumber : public InputField {
+      public:
+        InputNumber(
+          const char * fname,
+          int min, int max,
+          int value = 0,
+          DisableCodes disabled = Enabled,
+          const char * attr = NULL
+        );
+        virtual ~InputNumber() {}
+      protected:
+        InputNumber(
+          const char * type,
+          const char * fname,
+          int min, int max,
+          int value,
+          DisableCodes disabled,
+          const char * attr
+        );
+        virtual void AddAttr(PHTML & html) const;
+      private:
+        void Construct(int min, int max, int value);
+        int minValue, maxValue, initValue;
+    };
+
+    class InputRange : public InputNumber {
       public:
         InputRange(
           const char * fname,
@@ -911,10 +936,6 @@ class PHTML : public PStringStream
           const char * attr = NULL
         );
         virtual ~InputRange() {}
-      protected:
-        virtual void AddAttr(PHTML & html) const;
-      private:
-        int minValue, maxValue, initValue;
     };
 
     class InputFile : public InputField {
