@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23992 $
+ * $Revision: 24410 $
  * $Author: rjongbloed $
- * $Date: 2010-01-30 17:14:26 -0600 (Sat, 30 Jan 2010) $
+ * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
  */
 
 #ifndef PTLIB_PURL_H
@@ -392,7 +392,7 @@ class PURLLegacyScheme : public PURLScheme
       , defaultPort           (port)
     { }
 
-    bool Parse(const PString & url, PURL & purl) const
+    PBoolean Parse(const PString & url, PURL & purl) const
     { return purl.LegacyParse(url, this); }
 
     PString AsString(PURL::UrlFormat fmt, const PURL & purl) const
