@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20624 $
+ * $Revision: 23897 $
  * $Author: rjongbloed $
- * $Date: 2008-07-26 01:56:27 +0200 (sam 26 jui 2008) $
+ * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
  */
 
-#ifndef _PSOCKETS
-#define _PSOCKETS
+#ifndef PTLIB_SOCKET_H
+#define PTLIB_SOCKET_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -52,8 +52,8 @@ PLIST(PSocketList, PSocket);
 /**A network communications channel. This is based on the concepts in the
    Berkley Sockets library.
 
-   A socket represents a bidirectional communications channel to a \it port
-   at a remote \it host.
+   A socket represents a bidirectional communications channel to a <i>port</i>
+   at a remote <i>host</i>.
  */
 class PSocket : public PChannel
 {
@@ -485,11 +485,15 @@ class P_fd_set {
         return set;
       }
 
-  private:
+  protected:
     void Construct();
 
     SOCKET max_fd;
     fd_set * set;
+
+  private:
+    P_fd_set(const P_fd_set &) {}
+    void operator=(const P_fd_set &) {}
 };
 
 
@@ -537,6 +541,8 @@ class PWinSock : public PSocket
 };
 #endif
 
-#endif
+
+#endif // PTLIB_SOCKET_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
