@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23956 $
+ * $Revision: 24410 $
  * $Author: rjongbloed $
- * $Date: 2010-01-21 01:34:45 -0600 (Thu, 21 Jan 2010) $
+ * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
  */
 
 #ifdef __GNUC__
@@ -1570,7 +1570,7 @@ static void SplitCmdAndArgs(const PString & text, PINDEX pos, PCaselessString &
 }
 
 
-PBoolean PServiceHTML::ProcessMacros(PHTTPRequest & request,
+bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
                                  PString & text,
                                  const PString & defaultFile,
                                  unsigned options)
