@@ -23,9 +23,9 @@
  *
  * Contributor(s): Federico Pinna and Reitek S.p.A. (SASL authentication)
  *
- * $Revision: 23891 $
+ * $Revision: 24410 $
  * $Author: rjongbloed $
- * $Date: 2009-12-22 01:57:21 -0600 (Tue, 22 Dec 2009) $
+ * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
  */
 
 #ifdef __GNUC__
@@ -223,7 +223,7 @@ PBoolean PSMTPClient::BeginMessage(const PString & from,
 }
 
 
-PBoolean PSMTPClient::InternalBeginMessage()
+bool PSMTPClient::InternalBeginMessage()
 {
   PString localHost;
   PString peerHost;
