@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23337 $
+ * $Revision: 24359 $
  * $Author: rjongbloed $
- * $Date: 2009-09-04 20:21:09 -0500 (Fri, 04 Sep 2009) $
+ * $Date: 2010-05-18 03:07:12 -0500 (Tue, 18 May 2010) $
  */
 
 #ifdef __GNUC__
@@ -242,7 +242,7 @@ bool PVideoOutputDevice_SDL::InitialiseSDL()
 bool PVideoOutputDevice_SDL::ProcessSDLEvents()
 {
   if (screen == NULL || overlay == NULL) {
-    PTRACE(6, "VSDL\t Screen and/or overlay not open, so dont process events");
+    PTRACE(6, "VSDL\t Screen and/or overlay not open, so don't process events");
     return false;
   }
 
