@@ -24,9 +24,9 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 21764 $
+ * $Revision: 23962 $
  * $Author: rjongbloed $
- * $Date: 2008-12-10 07:44:14 +0000 (Wed, 10 Dec 2008) $
+ * $Date: 2010-01-21 22:39:06 -0600 (Thu, 21 Jan 2010) $
  */
 
 #include <ptlib.h>
@@ -45,102 +45,15 @@
 #pragma comment(lib, "gdi32.lib")
 #endif
 
-namespace PWLibStupidLinkerHacks {
-   int loadVideoForWindowsStuff;
-};
     
+#include <vfw.h>
+
 #ifdef __MINGW32__
 
 #define VHDR_DONE       0x00000001
 #define VHDR_KEYFRAME   0x00000008
-
-typedef struct videohdr_tag {
-    LPBYTE      lpData;
-    DWORD       dwBufferLength;
-    DWORD       dwBytesUsed;
-    DWORD       dwTimeCaptured;
-    DWORD       dwUser;
-    DWORD       dwFlags;
-    DWORD       dwReserved[4];
-} *LPVIDEOHDR;
-
-typedef struct tagCapDriverCaps {
-    UINT        wDeviceIndex;
-    BOOL        fHasOverlay;
-    BOOL        fHasDlgVideoSource;
-    BOOL        fHasDlgVideoFormat;
-    BOOL        fHasDlgVideoDisplay;
-    BOOL        fCaptureInitialized;
-    BOOL        fDriverSuppliesPalettes;
-    HANDLE      hVideoIn;
-    HANDLE      hVideoOut;
-    HANDLE      hVideoExtIn;
-    HANDLE      hVideoExtOut;
-} CAPDRIVERCAPS, *LPCAPDRIVERCAPS;
-
-typedef struct tagCaptureParms {
-    DWORD       dwRequestMicroSecPerFrame;  // Requested capture rate
-    BOOL        fMakeUserHitOKToCapture;    // Show "Hit OK to cap" dlg?
-    WORD        wPercentDropForError;       // Give error msg if > (10%)
-    BOOL        fYield;                     // Capture via background task?
-    DWORD       dwIndexSize;                // Max index size in frames (32K)
-    WORD        wChunkGranularity;          // Junk chunk granularity (2K)
-    BOOL        fUsingDOSMemory;            // Use DOS buffers?
-    WORD        wNumVideoRequested;         // # video buffers, If 0, autocalc
-    BOOL        fCaptureAudio;              // Capture audio?
-    WORD        wNumAudioRequested;         // # audio buffers, If 0, autocalc
-    WORD        vKeyAbort;                  // Virtual key causing abort
-    BOOL        fAbortLeftMouse;            // Abort on left mouse?
-    BOOL        fAbortRightMouse;           // Abort on right mouse?
-    BOOL        fLimitEnabled;              // Use wTimeLimit?
-    WORD        wTimeLimit;                 // Seconds to capture
-    BOOL        fMCIControl;                // Use MCI video source?
-    BOOL        fStepMCIDevice;             // Step MCI device?
-    DWORD       dwMCIStartTime;             // Time to start in MS
-    DWORD       dwMCIStopTime;              // Time to stop in MS
-    BOOL        fStepCaptureAt2x;           // Perform spatial averaging 2x
-    WORD        wStepCaptureAverageFrames;  // Temporal average n Frames
-    DWORD       dwAudioBufferSize;          // Size of audio bufs (0 = default)
-    BOOL        fDisableWriteCache;         // Attempt to disable write cache
-} CAPTUREPARMS, FAR *LPCAPTUREPARMS;
-
-typedef struct tagCapStatus {
-    UINT        uiImageWidth;               // Width of the image
-    UINT        uiImageHeight;              // Height of the image
-    BOOL        fLiveWindow;                // Now Previewing video?
-    BOOL        fOverlayWindow;             // Now Overlaying video?
-    BOOL        fScale;                     // Scale image to client?
-    POINT       ptScroll;                   // Scroll position
-    BOOL        fUsingDefaultPalette;       // Using default driver palette?
-    BOOL        fAudioHardware;             // Audio hardware present?
-    BOOL        fCapFileExists;             // Does capture file exist?
-    DWORD       dwCurrentVideoFrame;        // # of video frames cap'td
-    DWORD       dwCurrentVideoFramesDropped;// # of video frames dropped
-    DWORD       dwCurrentWaveSamples;       // # of wave samples cap'td
-    DWORD       dwCurrentTimeElapsedMS;     // Elapsed capture duration
-    HPALETTE    hPalCurrent;                // Current palette in use
-    BOOL        fCapturingNow;              // Capture in progress?
-    DWORD       dwReturn;                   // Error value after any operation
-    WORD        wNumVideoAllocated;         // Actual number of video buffers
-    WORD        wNumAudioAllocated;         // Actual number of audio buffers
-} CAPSTATUS, FAR *LPCAPSTATUS;
-
-#define WM_CAP_START                    WM_USER
-#define WM_CAP_SET_CALLBACK_ERROR       (WM_CAP_START+  2)
-#define WM_CAP_SET_CALLBACK_FRAME       (WM_CAP_START+  5)
-#define WM_CAP_SET_CALLBACK_VIDEOSTREAM (WM_CAP_START+  6)
-#define WM_CAP_GET_USER_DATA            (WM_CAP_START+  8)
-#define WM_CAP_SET_USER_DATA            (WM_CAP_START+  9)
-#define WM_CAP_DRIVER_CONNECT           (WM_CAP_START+  10)
-#define WM_CAP_DRIVER_DISCONNECT        (WM_CAP_START+  11)
-#define WM_CAP_DRIVER_GET_CAPS          (WM_CAP_START+  14)
-#define WM_CAP_GET_VIDEOFORMAT          (WM_CAP_START+  44)
-#define WM_CAP_SET_VIDEOFORMAT          (WM_CAP_START+  45)
-#define WM_CAP_SET_PREVIEW              (WM_CAP_START+  50)
-#define WM_CAP_GET_STATUS               (WM_CAP_START+  54)
-#define WM_CAP_GRAB_FRAME_NOSTOP        (WM_CAP_START+  61)
-#define WM_CAP_SET_SEQUENCE_SETUP       (WM_CAP_START+  64)
-#define WM_CAP_GET_SEQUENCE_SETUP       (WM_CAP_START+  65)
+#define WM_USER 1024
+#define WM_CAP_FIRSTA (WM_USER)
 
 #define capSetCallbackOnError(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_ERROR, 0, (LPARAM)(LPVOID)(fpProc)))
 #define capSetCallbackOnFrame(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_FRAME, 0, (LPARAM)(LPVOID)(fpProc)))
@@ -160,24 +73,15 @@ typedef struct tagCapStatus {
 #define capCaptureGetSetup(hwnd, s, wSize)         ((BOOL)::SendMessage(hwnd, WM_CAP_GET_SEQUENCE_SETUP, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPTUREPARMS)(s)))
 
 extern "C" {
-HWND VFWAPI capCreateCaptureWindowA (LPCSTR lpszWindowName, DWORD dwStyle,
-             int x, int y, int nWidth, int nHeight,
-             HWND hwndParent, int nID);
 #ifdef _MSC_VER
 BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
               int cbName, LPSTR lpszVer, int cbVer);
 #endif
 }
 
-#define capGetDriverDescription capGetDriverDescriptionA
-#define capCreateCaptureWindow capCreateCaptureWindowA
-
-#else // __MINGW32
-
-  #include <vfw.h>
-
 #endif // __MINGW32
 
+
 #define STEP_GRAB_CAPTURE 1
 
 
@@ -1155,6 +1059,17 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
       int & y   // Y position of device surface
     ) const;
 
+    /**Set the position of the output device, where relevant. For devices such as
+       files, this does nothing. For devices such as Windows, this sets the
+       position of the window on the screen.
+       
+       Returns: TRUE if the position can be set.
+      */
+    virtual bool SetPosition(
+      int x,  // X position of device surface
+      int y   // Y position of device surface
+    );
+
     LRESULT WndProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
 
   protected:
@@ -1417,6 +1332,22 @@ PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 }
 
 
+bool PVideoOutputDevice_Window::SetPosition(int x, int y)
+{
+  if (m_hWnd != NULL) {
+    RECT rect;
+    rect.top = y;
+    rect.left = x;
+    rect.bottom = y;
+    rect.right = x;
+    ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
+    ::SetWindowPos(m_hWnd, HWND_TOP, x+(x-rect.left), y, 0, 0, SWP_NOSIZE);
+  }
+
+  return true;
+}
+
+
 void PVideoOutputDevice_Window::Draw(HDC hDC)
 {
   RECT rect;
