@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22065 $
+ * $Revision: 24380 $
  * $Author: rjongbloed $
- * $Date: 2009-02-15 11:57:26 +0000 (Sun, 15 Feb 2009) $
+ * $Date: 2010-05-26 00:16:02 -0500 (Wed, 26 May 2010) $
  */
 
 #pragma implementation "sockets.h"
@@ -40,7 +40,6 @@
 #pragma implementation "ethsock.h"
 #pragma implementation "qos.h"
 
-
 #include <ptlib.h>
 #include <ptlib/sockets.h>
 
@@ -67,6 +66,10 @@
 #include <netinet/if_ether.h>
 #endif
 
+#if defined(P_NETBSD)
+#include <ifaddrs.h>
+#endif
+
 #define ROUNDUP(a) \
         ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
 
@@ -215,7 +218,7 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX *
     if (errno != EPROTO)
       return ConvertOSError(-1);
 
-    PTRACE(3, "PWLib\tAccept on " << sock << " failed with EPROTO - retrying");
+    PTRACE(3, "PTLib\tAccept on " << sock << " failed with EPROTO - retrying");
   }
 #else
   return ConvertOSError(os_handle = SetNonBlocking(::accept(listener.GetHandle(), addr, (socklen_t *)size)));
@@ -333,8 +336,8 @@ PChannel::Errors PSocket::Select(SelectList & read,
       if (fds[0].IsPresent(unblockPipe)) {
         PTRACE(6, "PWLib\tSelect unblocked fd=" << unblockPipe);
         BYTE ch;
-        ::read(unblockPipe, &ch, 1);
-        lastError = Interrupted;
+        if (ConvertOSError(::read(unblockPipe, &ch, 1), lastError, osError))
+          lastError = Interrupted;
       }
     }
   }
@@ -471,6 +474,12 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
   PBYTEArray buffer;
   struct ifconf ifConf;
 
+#if defined(P_NETBSD)
+  struct ifaddrs *ifap, *ifa;
+
+  PAssert(getifaddrs(&ifap) == 0, "getifaddrs failed");
+  for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
+#else
 #ifdef SIOCGIFNUM
   int ifNum;
   PAssert(::ioctl(sock.GetHandle(), SIOCGIFNUM, &ifNum) >= 0, "could not do ioctl for ifNum");
@@ -486,8 +495,14 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
     ifreq * ifName = ifConf.ifc_req;
 
     while (ifName < ifEndList) {
+#endif
       struct ifreq ifReq;
+#if !defined(P_NETBSD)
       memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+      memset(&ifReq, 0, sizeof(ifReq));
+      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
       
       if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
         int flags = ifReq.ifr_flags;
@@ -499,14 +514,16 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
         }
       }
       
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
       // move the ifName pointer along to the next ifreq entry
       ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
-#else
+#elif !defined(P_NETBSD)
       ifName++;
 #endif
     }
+#if !defined(P_NETBSD)
   }
+#endif
   
   return PFalse;
 }
@@ -575,7 +592,7 @@ PBoolean PSocket::os_recvfrom(
   // read a packet 
   int r = ::recvmsg(os_handle, &readData, flags);
   if (r == -1) {
-    PTRACE(5, "PWLIB\trecvmsg returned error " << r);
+    PTRACE(5, "PTLIB\trecvmsg returned error " << r);
     ::recvmsg(os_handle, &readData, MSG_ERRQUEUE);
   }
 
@@ -1148,10 +1165,10 @@ PBoolean process_rtentry(struct rt_msghdr *rtm, char *ptr, unsigned long *p_net_
   if ((~rtm->rtm_flags&RTF_LLINFO)
 #if defined(P_NETBSD) || defined(P_QNX)
         && (~rtm->rtm_flags&RTF_CLONED)     // Net BSD has flag one way
-#elif !defined(P_OPENBSD)
-        && (~rtm->rtm_flags&RTF_WASCLONED)  // Free BSD/MAC has it another
+#elif !defined(P_OPENBSD) && !defined(P_FREEBSD)
+        && (~rtm->rtm_flags&RTF_WASCLONED)  // MAC has it another
 #else
-                                            // Open BSD does not have it at all!
+                                            // Open/Free BSD does not have it at all!
 #endif
      ) {
 
@@ -1521,16 +1538,147 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #endif
 
 
-bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
-{
-  if (cancellation != NULL)
-    return cancellation->Wait(timeout);
 
-  // Need an implementation here!!!!!!!
-  PThread::Sleep(timeout);
-  return false;
+#ifdef P_HAS_NETLINK
+
+#include <asm/types.h>
+#include <sys/socket.h>
+#include <linux/netlink.h>
+#include <linux/rtnetlink.h>
+#include <linux/genetlink.h>
+
+#include <memory.h>
+#include <errno.h>
+
+class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
+{
+  public:
+    int m_fdLink;
+    int m_fdCancel[2];
+
+    NetLinkRouteTableDetector()
+    {
+      m_fdLink = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+
+      if (m_fdLink != -1) {
+        struct sockaddr_nl sanl;
+        memset(&sanl, 0, sizeof(sanl));
+        sanl.nl_family = AF_NETLINK;
+        sanl.nl_groups = RTMGRP_LINK | RTMGRP_IPV4_IFADDR;
+
+        bind(m_fdLink, (struct sockaddr *)&sanl, sizeof(sanl));
+      }
+
+      if (pipe(m_fdCancel) == -1)
+        m_fdCancel[0] = m_fdCancel[1] = -1;
+
+      PTRACE(3, "PTLIB\tOpened NetLink socket");
+    }
+
+    ~NetLinkRouteTableDetector()
+    {
+      if (m_fdLink != -1)
+        close(m_fdLink);
+      if (m_fdCancel[0] != -1)
+        close(m_fdCancel[0]);
+      if (m_fdCancel[1] != -1)
+        close(m_fdCancel[1]);
+    }
+
+    bool Wait(const PTimeInterval & timeout)
+    {
+      if (m_fdCancel[0] == -1)
+        return false;
+
+      bool ok = true;
+      while (ok) {
+        fd_set fds;
+        FD_ZERO(&fds);
+        FD_SET(m_fdCancel[0], &fds);
+
+        struct timeval tval;
+        struct timeval * ptval = NULL;
+        if (m_fdLink != -1) {
+          tval.tv_sec  = timeout.GetMilliSeconds() / 1000;
+          tval.tv_usec = (timeout.GetMilliSeconds() % 1000) * 1000;
+          ptval = &tval;
+
+          FD_SET(m_fdLink, &fds);
+        }
+
+        int result = select(std::max(m_fdLink, m_fdCancel[0])+1, &fds, NULL, NULL, ptval);
+        if (result < 0)
+          return false;
+        if (result == 0)
+          return true;
+
+        if (FD_ISSET(m_fdCancel[0], &fds))
+          return false;
+
+        struct sockaddr_nl snl;
+        char buf[4096];
+        struct iovec iov = { buf, sizeof buf };
+        struct msghdr msg = { (void*)&snl, sizeof snl, &iov, 1, NULL, 0, 0};
+
+        int status = recvmsg(m_fdLink, &msg, 0);
+        if (status < 0)
+          return false;
+
+        for (struct nlmsghdr * nlmsg = (struct nlmsghdr *)buf;
+             NLMSG_OK(nlmsg, (unsigned)status);
+             nlmsg = NLMSG_NEXT(nlmsg, status)) {
+          if (nlmsg->nlmsg_len < sizeof(struct nlmsghdr))
+            break;
+
+          switch (nlmsg->nlmsg_type) {
+            case RTM_NEWADDR :
+            case RTM_DELADDR :
+              PTRACE(3, "PTLIB\tInterface table change detected via NetLink");
+              return true;
+          }
+        }
+      }
+      return false;
+    }
+
+    void Cancel()
+    {
+      PAssert(write(m_fdCancel[1], "", 1) == 1, POperatingSystemError);
+    }
+};
+
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new NetLinkRouteTableDetector();
 }
 
+#else
+
+class DummyRouteTableDetector : public PIPSocket::RouteTableDetector
+{
+  public:
+    bool Wait(const PTimeInterval & timeout)
+    {
+      return !m_cancel.Wait(timeout);
+    }
+
+    void Cancel()
+    {
+      m_cancel.Signal();
+    }
+
+  private:
+    PSyncPoint m_cancel;
+};
+
+
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new DummyRouteTableDetector();
+}
+
+#endif
+
 
 // fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
 // 00000000000000000000000000000001 01 80 10 80       lo
@@ -1574,7 +1722,13 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
   PBYTEArray buffer;
   struct ifconf ifConf;
   
+#if defined(P_NETBSD)
+  struct ifaddrs *ifap, *ifa;
 
+  PAssert(getifaddrs(&ifap) == 0, "getifaddrs failed");
+
+  for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
+#else
   // HERE
 #if defined(SIOCGIFNUM)
   int ifNum;
@@ -1590,9 +1744,14 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
     void * ifEndList = (char *)ifConf.ifc_req + ifConf.ifc_len;
     ifreq * ifName = ifConf.ifc_req;
     while (ifName < ifEndList) {
-
+#endif
       struct ifreq ifReq;
+#if !defined(P_NETBSD)
       memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+      memset(&ifReq, 0, sizeof(ifReq));
+      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
 
       if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
         int flags = ifReq.ifr_flags;
@@ -1608,13 +1767,25 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
           }
 #endif
 
+#if !defined(P_NETBSD)
           memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+          memset(&ifReq, 0, sizeof(ifReq));
+          strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
+
           if (ioctl(sock.GetHandle(), SIOCGIFADDR, &ifReq) >= 0) {
 
             sockaddr_in * sin = (sockaddr_in *)&ifReq.ifr_addr;
             PIPSocket::Address addr = sin->sin_addr;
 
+#if !defined(P_NETBSD)
             memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+            memset(&ifReq, 0, sizeof(ifReq));
+            strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
+
             if (ioctl(sock.GetHandle(), SIOCGIFNETMASK, &ifReq) >= 0) {
               PIPSocket::Address mask = 
 #ifndef P_BEOS
@@ -1652,15 +1823,17 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
         }
       }
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
       // move the ifName pointer along to the next ifreq entry
       ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
-#else
+#elif !defined(P_NETBSD)
       ifName++;
 #endif
 
     }
+#if !defined(P_NETBSD)
   }
+#endif
   return PTrue;
 }
 
